# The Lab Evaluation Method was implemented by Attila von Thermann and is part of the publication "Evaluating Peripheral Interaction": http://www.medien.ifi.lmu.de/forschung/publikationen/detail?pub=hausen2014tei

import sys,pygame,random,datetime,time,SecTask,PriTask,Util,csv
from pygame.rect import *
from pygame.locals import *

#--------------------------------------------
#START VARIABLES
username = "P1"								#Username for Log Data
#taskMode = Util.TRAINING_MODE_C
#taskMode = Util.BASELINE_MODE_1
taskMode = Util.BASELINE_MODE_3				#Event-Based Tasks, other modes can be tried but are not properly tested
priTaskType = PriTask.JUST_VISUAL_CONT      #can be JUST_VISUAL (used for Event-Based Task( or VISUAL_AND_AUDIO. will be overwritten by mapping
secTaskType = SecTask.NO_TASK           	#can be NO_TASK or TASK_1 or TASK_2. will be overwritten by mapping; can be used to also control the secondary task


#--------------------------------------------
#TUNING VARIABLES
screenwidth=667						
screenwidthbonus=80
screenheight=500                       
symbolsize=50                       		#size (height and width) of the symbols in pixels.    standard is 50
maxTestTime=300.0                   			#the time test takes in seconds.                      standard is 300
endTimeForThinking = 3.0            		#timespan at the end where no new items appear        standard is 3.0

###Variables for Event-Based Task
hitboxbonus=3                   			# for PT3: help for user
pt3objectCreationThreshold= 21
maxObjectsPt3 = 30							#Number of objects on screen or each round
pt3goodJobTime = 0.5


###Part of other - non event based tasks - usually no need to change them
baseRoundTime_p=1.5                 #minimum time for a round (where items appear or disappear).                               standard is 1.5
maxRoundExtraTime_p=0.5             #maximum time bonus for a round (a random portion of this bonus is added to round time)    standard is 0.5
baseRoundTime_s=2.7                 #minimum time for a round (where items appear or disappear)                                standard is 2.4
maxRoundExtraTime_s=1.0             #maximum time bonus for a round (a random portion of this bonus is added to round time)    standard is 1.0
estimatedMaxReactionTime_p=4.0      #estimated time for reaction to event. if button press comes later, evaluation will not recognize. standard is 4.0
maxErrorsBeforeSearchCancels_p=4    #user can press x wrong buttons and evaluation will still find his x+1st button press              standard is 4
estimatedMaxReactionTime_s=9.0      #estimated time for reaction to event. only the last event in the time frame will be recognized    standard is 7.0
#PRIMARY: the chance that an object will stay alive when its lifetime>index
survivalPercentageInRound=[100,96,92,90,85,80,75,70,60,50,20,10,5,2,1,0]
#PRIMARY: the chance that a new object is created when amount>index are already existing
creationPercentageWhenObjects=[60,40,30,10,0]
#SECONDARY:
#changePercentageInRound=[0,0,3,5,8,10,12,15,20,25,30,35,40,45,50,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100]



#--------------------------------------------
#DEBUGGING VARIABLES
debug = False
debug_buttons = False
debug_rounds = False
debug_changes = False
debug_lights = False
debug_secInput = False
debug_eval = False


#--------------------------------------------
#MAPPING TASKMODES

taskModeString = ""
if taskMode == Util.TASK_MODE_A:
    priTaskType = PriTask.JUST_VISUAL
    secTaskType = SecTask.TASK_1
    taskModeString = "MODE_A"
    
elif taskMode == Util.TASK_MODE_B:
    priTaskType = PriTask.JUST_VISUAL_CONT
    secTaskType = SecTask.TASK_2
    taskModeString = "MODE_B"
    
elif taskMode == Util.TASK_MODE_C:
    priTaskType = PriTask.JUST_VISUAL
    secTaskType = SecTask.TASK_2
    taskModeString = "MODE_C"
    
elif taskMode == Util.TASK_MODE_D:
    priTaskType = PriTask.JUST_VISUAL_CONT
    secTaskType = SecTask.TASK_1
    taskModeString = "MODE_D"
    
elif taskMode == Util.TASK_MODE_E:
    priTaskType = PriTask.VISUAL_AND_AUDIO
    secTaskType = SecTask.TASK_1
    taskModeString = "MODE_E"
    
elif taskMode == Util.TASK_MODE_F:
    priTaskType = PriTask.VISUAL_AND_AUDIO
    secTaskType = SecTask.TASK_2
    taskModeString = "MODE_F"
    
elif taskMode == Util.BASELINE_MODE_1:
    priTaskType = PriTask.JUST_VISUAL
    secTaskType = SecTask.NO_TASK
    taskModeString = "BASELINE_1"
    
elif taskMode == Util.BASELINE_MODE_3:
    priTaskType = PriTask.JUST_VISUAL_CONT
    secTaskType = SecTask.NO_TASK
    taskModeString = "BASELINE_3"
    
elif taskMode == Util.TRAINING_MODE_A:
    priTaskType = PriTask.JUST_VISUAL
    secTaskType = SecTask.TASK_1
    taskModeString = "TRAINING_A"
    
elif taskMode == Util.TRAINING_MODE_B:
    priTaskType = PriTask.JUST_VISUAL_CONT
    secTaskType = SecTask.TASK_2
    taskModeString = "TRAINING_B"
    
elif taskMode == Util.TRAINING_MODE_C:
    priTaskType = PriTask.JUST_VISUAL
    secTaskType = SecTask.TASK_2
    taskModeString = "TRAINING_C"
    
elif taskMode == Util.TRAINING_MODE_D:
    priTaskType = PriTask.JUST_VISUAL_CONT
    secTaskType = SecTask.TASK_1
    taskModeString = "TRAINING_D"
    
elif taskMode == Util.TRAINING_MODE_E:
    priTaskType = PriTask.VISUAL_AND_AUDIO
    secTaskType = SecTask.TASK_1
    taskModeString = "TRAINING_E"
    
elif taskMode == Util.TRAINING_MODE_F:
    priTaskType = PriTask.VISUAL_AND_AUDIO
    secTaskType = SecTask.TASK_2
    taskModeString = "TRAINING_F"
    
elif taskMode == Util.TRAINING_MODE_1:
    priTaskType = PriTask.JUST_VISUAL
    secTaskType = SecTask.NO_TASK
    taskModeString = "TRAIN_1"
    
elif taskMode == Util.TRAINING_MODE_2:
    priTaskType = PriTask.VISUAL_AND_AUDIO
    secTaskType = SecTask.NO_TASK
    taskModeString = "TRAIN_2"
    
elif taskMode == Util.TRAINING_MODE_3:
    priTaskType = PriTask.JUST_VISUAL_CONT
    secTaskType = SecTask.NO_TASK
    taskModeString = "TRAIN_3"
else:
    print "ERROR AT MAPPING TASK MODES TO TASKS"




# shorten the test if it is in training mode
if taskMode > 10:
    maxTestTime=60.0
    
# shorten the test if it is in debug/training mode
if taskMode > 20:
    maxTestTime=20.0



#--------------------------------------------
#ALGORITHM VARIABLES

greycolor_rgba = pygame.Color(180,180,180,255)
whitecolor_rgba = pygame.Color(255,255,255,1)
greycolor = pygame.Color(180,180,180)
violetcolor = pygame.Color(227,83,227)
bluecolor = pygame.Color(0,0,255)
greencolor = pygame.Color(26,146,67)
pt3lastKey = 0
pt3dominantColour = 999
maxPTobjects = 5
pt3errorCount = 0
pt3hitCount = 0
buttonPressed = False
pt3lastHit = 0.0
pt3reactionTimesList = []
pt3goodJobIsThere = False
numLockStateIsIncorrect = False
errorTimesList = []
hitTimesList = []

if priTaskType == PriTask.JUST_VISUAL_CONT:
    maxPTobjects = maxObjectsPt3


# check if test is already running
isTestRunning=False

# how many rounds will the relevant light be blocked from having changes after it was changed
secChangeBlockingRoundAmount = int(estimatedMaxReactionTime_s / baseRoundTime_s) + 1    

# save the time float of when the last round ended
endTimeOfLastRound_p=0
endTimeOfLastRound_s=0

# save the float of the current time
timeNow=0

# how long will the current round last? in the beginning, base round time + half the round time bonus
thisRoundLifeTime_p=baseRoundTime_p+(maxRoundExtraTime_p/2)
thisRoundLifeTime_s=baseRoundTime_s+(maxRoundExtraTime_s/2)

# count the round numbers for debugging
roundNumber_p=0
roundNumber_s=0

# log if there are changes, so display is only updated when there are
changes = True

# in case time doesnt start with 0 but with date and time
startClock = 0

# better performance by just checking once if there is a secondary task and save it in a bool
isThereSecTask = True

# init a blank object in case there is no secondary task
secTaskManager = ""

# find out if program runs fast enough... every time the while loop runs through without having a "round", this counts up
performanceMeasure = 0





#--------------------------------------------
# initialize symbol png files and sectask bool variable

if secTaskType == SecTask.NO_TASK: isThereSecTask = False


if priTaskType == PriTask.JUST_VISUAL or PriTask.JUST_VISUAL_CONT:
    symbolSurfaces=[0,pygame.image.load("res/symbol_bc.png"),pygame.image.load("res/symbol_bs.png"),pygame.image.load("res/symbol_bt.png"),
                pygame.image.load("res/symbol_gc.png"),pygame.image.load("res/symbol_gs.png"),pygame.image.load("res/symbol_gt.png"),
                pygame.image.load("res/symbol_vc.png"),pygame.image.load("res/symbol_vs.png"),pygame.image.load("res/symbol_vt.png")]

elif priTaskType == PriTask.VISUAL_AND_AUDIO:
    symbolSurfaces=[pygame.image.load("res/symbol_bc.png"),pygame.image.load("res/symbol_gs.png"),pygame.image.load("res/symbol_vt.png")]



#--------------------------------------------
#build string for foldername of folder where txt data is stored
foldername = Util.buildFoldernameString(datetime.datetime.now(),username,taskModeString)



#--------------------------------------------
# pygame display initialization
pygame.init()

random.seed()

screen = pygame.display.set_mode(Rect(0,0,screenwidth+screenwidthbonus,screenheight).size)
    
    
pygame.display.set_caption("Peripheral Lab Study")
mainsurface = pygame.display.get_surface()

bgcolor = greycolor

mainsurface.fill(bgcolor)

background_surf = pygame.image.load("res/bg.png")
background_surf_beforestart = pygame.image.load("res/bg_bs.png")


# initialize sound if there is a primary task with sound
if priTaskType == PriTask.VISUAL_AND_AUDIO:
    pygame.mixer.init()



#--------------------------------------------
# initialize task managers for primary and secondary

primaryTaskManager = PriTask.PrimaryTaskManager(priTaskType,foldername,debug,screenwidth,screenheight,symbolsize,
                                                creationPercentageWhenObjects,survivalPercentageInRound,hitboxbonus,maxObjectsPt3)

# if isThereSecTask:
#     secTaskManager = SecTask.SecondaryTaskManager(secTaskType, portName, changePercentageInRound, debug_lights, lightness, debug_secInput,
#                                                   secChangeBlockingRoundAmount)




#--------------------------------------------
#display while before start
while (not isTestRunning):
    mainsurface.fill(bgcolor)
    mainsurface.blit(background_surf_beforestart, (0,0))
    for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            elif event.type == KEYDOWN:
                if event.key == K_RETURN or event.key == pygame.K_KP_ENTER:
                    isTestRunning = True
                    startClock = time.clock()
                    
    pygame.display.update()



#--------------------------------------------
#clear window when test starts
mainsurface.blit(background_surf, (0,0))
pygame.display.update()

pt3goodJobFont = ""
pt3hitCountFont = ""


if priTaskType == PriTask.JUST_VISUAL_CONT:
    pt3goodJobFont = pygame.font.SysFont("Courier New", 15, False, False)
    pt3hitCountFont = pygame.font.SysFont("Courier New", 32, False, False)




#--------------------------------------------
#display while loop once the test is running
while isTestRunning:
    timeNow = time.clock() - startClock
    changes = False
    isItARound = False
 

    #check if maxTestTime is reached, if it is -> set isTestRunning false
    if timeNow > maxTestTime:
        isTestRunning = False
        
        

    #check for primary task user input, write to user text file
    for event in pygame.event.get():
            if event.type == KEYUP:
                buttonPressed = False
            if event.type == KEYDOWN:
                if event.key == K_KP1:
                    if debug or debug_buttons: print "Button 1 pressed at " + str(timeNow)
                    pt3lastKey = 1
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,1)
                            
                elif event.key == K_KP2:
                    if debug or debug_buttons: print "Button 2 pressed at " + str(timeNow)
                    pt3lastKey = 2
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,2)
                            
                elif event.key == K_KP3:
                    if debug or debug_buttons: print "Button 3 pressed at " + str(timeNow)
                    pt3lastKey = 3
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,3)
                            
                elif event.key == K_KP4:
                    if debug or debug_buttons: print "Button 4 pressed at " + str(timeNow)
                    pt3lastKey = 4
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,4)
                            
                elif event.key == K_KP5:
                    if debug or debug_buttons: print "Button 5 pressed at " + str(timeNow)
                    pt3lastKey = 5
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,5)
                            
                elif event.key == K_KP6:
                    if debug or debug_buttons: print "Button 6 pressed at " + str(timeNow)
                    pt3lastKey = 6
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,6)
                            
                elif event.key == K_KP7:
                    if debug or debug_buttons: print "Button 7 pressed at " + str(timeNow)
                    pt3lastKey = 7
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,7)
                            
                elif event.key == K_KP8:
                    if debug or debug_buttons: print "Button 8 pressed at " + str(timeNow)
                    pt3lastKey = 8
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,8)
                            
                elif event.key == K_KP9:
                    if debug or debug_buttons: print "Button 2 pressed at " + str(timeNow)
                    pt3lastKey = 9
                    if priTaskType == PriTask.JUST_VISUAL_CONT:
                        buttonPressed = True
                        if ((pt3lastKey-1)/3) + 1 != pt3dominantColour:
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    else:
                        primaryTaskManager.writeUserEvent(timeNow,9)
                            
            elif event.type == MOUSEBUTTONDOWN:
                #print "buttonpressed is " + str(buttonPressed)
                if buttonPressed and priTaskType == PriTask.JUST_VISUAL_CONT:
                    if ((pt3lastKey-1)/3) + 1 == pt3dominantColour:
                        if primaryTaskManager.mouseButtonClick(pygame.mouse.get_pos(),pt3lastKey):
                            changes = True
                            pt3hitCount = pt3hitCount + 1
                            
                            
                            tempReactionTime = timeNow - pt3lastHit
                            pt3reactionTimesList.append(tempReactionTime)
                            hitTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                            
                            pt3lastHit = timeNow
                            
                        else:
                            if debug: print "mousebuttonclick not true!"
                            pt3errorCount = pt3errorCount + 1
                            errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                            #buttonPressed = False
                    else:
                        if debug: print "not dominant colour"
                        pt3errorCount = pt3errorCount + 1
                        errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                        #buttonPressed = False
                else:
                    pt3errorCount = pt3errorCount + 1
                    errorTimesList.append(datetime.datetime.now().strftime("%H:%M:%S.%f"))
                    
    #check for secondary task user input, write to user text file                
    if isThereSecTask:
        secondaryUserInput = secTaskManager.checkForUserInput()
        
        if secondaryUserInput == 100:
            isThereSecTask = True   #something has to be written into this line, so a true bool is set to true
            #print "" + "nothing has been turned"
        elif  secondaryUserInput == SecTask.RED:
            primaryTaskManager.writeUserEvent(timeNow,10 + SecTask.RED)
            if debug_secInput: print "" + "main light has been turned to red"
        elif  secondaryUserInput == SecTask.GREEN:
            primaryTaskManager.writeUserEvent(timeNow,10 + SecTask.GREEN)
            if debug_secInput: print "" + "main light has been turned to green"
        elif  secondaryUserInput == SecTask.BLUE:
            primaryTaskManager.writeUserEvent(timeNow,10 + SecTask.BLUE)
            if debug_secInput: print "" + "main light has been turned to blue"
        else:
            print "ERROR WITH SECONDARY TASK EVENT RECOGNITION (returned int value != 100, red, green, blue"
        
        
        #SECONDARY ROUNDS
        #check if round is over. if yes -> a new round begins!
        if ((timeNow > endTimeOfLastRound_s + thisRoundLifeTime_s) and (timeNow < maxTestTime - endTimeForThinking)):
            isItARound = True
            roundNumber_s = roundNumber_s+1
            if debug or debug_rounds: print "------ \n New round (number " + str(roundNumber_s) + ") at " + str(timeNow)
            endTimeOfLastRound_s = timeNow
            
            #compute lifetime for the round that is now beginning
            thisRoundLifeTime_s = baseRoundTime_s + (random.randint(0, 100) * maxRoundExtraTime_s) / 100 
            
            lightChangeResult = secTaskManager.checkChangeCreation(timeNow)
            
            if lightChangeResult != 999:    # 999 is magic number for "nothing found"
                if debug_lights: print "lightChange result ist " + str(lightChangeResult)
                primaryTaskManager.writeBotEventFromSec(timeNow, lightChangeResult) #writes the program event to file
        
        #PERIPHERAL UPDATE
        secTaskManager.updateLights()
    
    
    
    # PRIMARY TASK 3
    if priTaskType == PriTask.JUST_VISUAL_CONT:
        
        if timeNow > pt3goodJobTime + pt3lastHit and pt3goodJobIsThere:
            pt3goodJobIsThere = False
            changes = True
            #print "good job is going down!"
        
        if primaryTaskManager.aliveObjects < pt3objectCreationThreshold:
            roundNumber_p = roundNumber_p +1
            # create objects:
            #print "trying to fill field with objects"
            # set new dominant colour
            # adapt creation threshold to new situation
            pt3creationResult = primaryTaskManager.fillWithObjects(timeNow)
            
            pt3dominantColour = pt3creationResult[0]
            pt3objectCreationThreshold = maxObjectsPt3 +1 - pt3creationResult[1]
            
            #print "field has been filled with objects"
            if pt3dominantColour == PriTask.PT3BLUE:
                bgcolor = bluecolor
            elif pt3dominantColour == PriTask.PT3GREEN:
                bgcolor = greencolor
            elif pt3dominantColour == PriTask.PT3VIOLET:
                bgcolor = violetcolor
            changes = True
            
            
            
            
    else:
        #PRIMARY ROUNDS, for pt 1 and 2
        #check if round is over. if yes -> a new round begins!
        if ((timeNow > endTimeOfLastRound_p + thisRoundLifeTime_p) and (timeNow < maxTestTime - endTimeForThinking)):
            isItARound = True
            roundNumber_p = roundNumber_p+1
            if debug or debug_rounds: print "------ \n New round (number " + str(roundNumber_p) + ") at " + str(timeNow)
            endTimeOfLastRound_p = timeNow
            
            #compute lifetime for the round that is now beginning
            thisRoundLifeTime_p = baseRoundTime_p + (random.randint(0, 100) * maxRoundExtraTime_p) / 100
        
            
            # check if new items will be created and maybe create them. return true if one is created
            changesCreation = primaryTaskManager.checkNewItemCreation(timeNow)
                            
            # check if old items disappear. return true if >0 do
            changesDeletion = primaryTaskManager.checkOldItemDeletion()
            
            # combine the two booleans to find out if display has to be updated
            changes = changesCreation or changesDeletion    
                
            if debug_changes: print str(roundNumber_p) + ": changes is " + str(changes) + ", creation is " + str(changesCreation) + ", deletion is " + str(changesDeletion)           
    
    
    
    #only update screen if round ends AND there are changes -> performance profit
    if changes:
        if debug_changes: print "There are changes! Now alive Objects: " + str(primaryTaskManager.getNumberOfAliveObjects())
        
        mainsurface.fill(bgcolor)
        mainsurface.blit(background_surf, (0,0))
        for i3 in range(0,maxPTobjects):
            if primaryTaskManager.findOutIfObjectIsAlive(i3):
                
                whichSurface = 0
                
                if priTaskType == PriTask.JUST_VISUAL or priTaskType == PriTask.JUST_VISUAL_CONT:
                    whichSurface = primaryTaskManager.getObjectType(i3)
                
                elif priTaskType == PriTask.VISUAL_AND_AUDIO:
                    whichSurface = (primaryTaskManager.getObjectType(i3) -1) / 3
                    
                mainsurface.blit(symbolSurfaces[whichSurface],primaryTaskManager.getObjectPosition(i3))
                
        if priTaskType == PriTask.JUST_VISUAL_CONT:
            
            #render the number of hits
            tempHitCountSurface = pt3hitCountFont.render(str(pt3hitCount),False,greycolor_rgba,whitecolor_rgba)
            mainsurface.blit(tempHitCountSurface,(screenwidth + screenwidthbonus / 4,(screenheight*3)/4))
            
            
            #render "good job" if there was a hit recently
            if pt3hitCount > 0  and timeNow < pt3goodJobTime + pt3lastHit:
                
                pt3goodJobIsThere = True
                #print "drawing good job at " + str(timeNow)
                tempGoodJobSurface = pt3goodJobFont.render("Good Job!",False,greycolor_rgba,whitecolor_rgba)
                mainsurface.blit(tempGoodJobSurface,(screenwidth,screenheight/2))
                
                
        pygame.display.update()
    else:
        if debug_changes: print "No Changes."
    
    if (not isItARound) and (not changes): performanceMeasure = performanceMeasure + 1



#--------------------------------------------
# finalize the text files needed for evaluation
primaryTaskManager.closeTxtFiles()

if isThereSecTask:
    secTaskManager.close()

background_surf = pygame.image.load("res/bg_ev.png")
pygame.display.update()




#--------------------------------------------
#evaluate results


print "\n\n-------\nEVALUATION BEGINS"

botEventList = []
userEventList = []
primaryTaskBotEventList = []
secondaryTaskBotEventList = []
primaryTaskUserEventList = []
secondaryTaskUserEventList = []
numberOfIncorrectUserEvents_p = 0
doneWithList = False
botLogFile = open(foldername + "/botLog.txt",'r')
userLogFile = open(foldername + "/userLog.txt",'r')
reactionTimesFile_p = open(foldername + "/reactionTimes_p.txt",'w')
reactionTimesFile_s = open(foldername + "/reactionTimes_s.txt",'w')
allResultsFile = csv.writer(open('txt/all_results.csv', 'a'), delimiter=';')
resultFile = csv.writer(open(foldername + '/results.csv', 'wb'), delimiter=';')
hitFile = csv.writer(open(foldername + '/hits.csv', 'wb'), delimiter=';')
errorFile = csv.writer(open(foldername + '/errors.csv', 'wb'), delimiter=';')

# read in program events from text file
while not doneWithList:
    line=botLogFile.readline()
    if line != "EOF":
        
        # time of event, type of event, xcoord, ycoord, hasItBeenCheckedYet
        
        newItem = [float(line),int(botLogFile.readline()),int(botLogFile.readline()),int(botLogFile.readline()),False]
        if isThereSecTask:
            if int(newItem[1]) < 99:
                primaryTaskBotEventList.append(newItem)
            else:
                secondaryTaskBotEventList.append(newItem)
        botEventList.append(newItem)
    else:
        doneWithList = True

doneWithList = False  

# read in user input events from text file
while not doneWithList:
    line=userLogFile.readline()
    if line != "EOF":
        
        # time of event, type of event, hasItBeenCheckedYet, is it a correct event
        
        newItem = [float(line),int(userLogFile.readline()),False,False]
        if isThereSecTask:
            itemCode = int(newItem[1])
            if itemCode < 10:
                primaryTaskUserEventList.append(newItem)
            else:
                secondaryTaskUserEventList.append(newItem)
        userEventList.append(newItem)
    else:
        doneWithList = True
        
print "Files have been read in. program events: " + str(len(botEventList)) + ", userEvents: " + str(len(userEventList))



# variables for final results

pri_task_name = "N.A." 
pri_PE_amount = "N.A."       # primary task: amount of program events
pri_UE_amount= "N.A."        # primary task: amount of user events
pri_CUE_amount = "N.A."      # primary task: amount of correct user events
pri_IPE_amount = "N.A."      # primary task: amount of incorrect program events = program events not correctly reacted to
pri_avg_reaction = "N.A."    # primary task: average reaction time
pri_sdv_reaction = "N.A."    # primary task: standard deviation of reaction times
pri_err_rate_1 = "N.A."      # primary task: error rate 1: percent of ( program events correctly reacted to / amount of all program events)
pri_err_rate_2 = "N.A."      # primary task: error rate 2: percent of ( correct user events / amount of all user events )

sec_task_name = "N.A." 
sec_PE_amount = "N.A."       # secondary task: amount of program events
sec_CUE_amount = "N.A."      # secondary task: amount of correct user events
sec_IPE_amount = "N.A."      # secondary task: amount of incorrect program events = program events not correctly reacted to
sec_avg_reaction = "N.A."    # secondary task: average reaction time
sec_sdv_reaction = "N.A."    # secondary task: standard deviation of reaction times
sec_err_rate_1 = "N.A."      # secondary task: error rate: percent of ( program events correctly reacted to / amount of all program events)
sec_event_Pquot = "N.A."     # secondary task: secondary program events / (secPE + priPE)



# fill first result variables with content

pri_task_name = "PRI_TASK_" + str(priTaskType)
if isThereSecTask:
    sec_task_name = "SEC_TASK_" + str(secTaskType)
else:
    sec_task_name = "NO_SEC_TASK"



if isThereSecTask:
    pri_PE_amount = len(primaryTaskBotEventList)
    pri_UE_amount = len(primaryTaskUserEventList)

else:
    pri_PE_amount = len(botEventList)
    pri_UE_amount = len(userEventList)


#sort out user events that happened before first symbol
if len(userEventList) > 0:
    userEventList = Util.sortOutEarlyEvents(botEventList[0][0],userEventList)

if debug_eval: print "primaryTaskBotEventList has elements: " + str(len(primaryTaskBotEventList))
if debug_eval: print "primaryTaskUserEventList has elements: " + str(len(primaryTaskUserEventList))

if isThereSecTask:
    
    if debug_eval: print "secondaryTaskBotEventList has elements: " + str(len(secondaryTaskBotEventList))
    if debug_eval: print "secondaryTaskUserEventList has elements: " + str(len(secondaryTaskUserEventList))
     
    if len(primaryTaskUserEventList) > 0:
        primaryTaskUserEventList    = Util.sortOutEarlyEvents(primaryTaskBotEventList[0][0],primaryTaskUserEventList)
        
        
    if len(secondaryTaskUserEventList) > 0:    
        secondaryTaskUserEventList  = Util.sortOutEarlyEvents(secondaryTaskBotEventList[0][0],secondaryTaskUserEventList)


# go through all program events and search for corresponding userEvents
# first we try to get the overall / primary result.
# with secondary task: only put primary event lists into SCUE-function
# without secondary task: there are no secondary events, so put them all in
if isThereSecTask:
    primaryResultSet = Util.searchCorrespondingUserEvent(debug_eval,True,False,primaryTaskBotEventList,
                                                         primaryTaskUserEventList,estimatedMaxReactionTime_p,maxErrorsBeforeSearchCancels_p)
else:
    primaryResultSet = Util.searchCorrespondingUserEvent(debug_eval,True,False,botEventList,userEventList,estimatedMaxReactionTime_p,
                                                         maxErrorsBeforeSearchCancels_p)

#map values from returned set to temporary variables
reactionTimesList_p                     = primaryResultSet[0]
totalReactionTime                       = primaryResultSet[1]
resultString                            = primaryResultSet[2]
incorrectEventsFound                    = primaryResultSet[3]
numberOfCorrectlyReactedBotEvents       = primaryResultSet[4]
numberOfBotEventsNotReactedToCorrectly  = primaryResultSet[5]


if priTaskType == PriTask.JUST_VISUAL_CONT:
    reactionTimesList_p = pt3reactionTimesList



# map values from temporary variables to final result variables

pri_CUE_amount = numberOfCorrectlyReactedBotEvents
pri_IPE_amount = numberOfBotEventsNotReactedToCorrectly


if priTaskType == PriTask.JUST_VISUAL_CONT:
    pri_avg_reaction = Util.computeAverage(pt3reactionTimesList)
    pri_PE_amount = roundNumber_p

else:
    if pri_CUE_amount > 0:
        #print "calculating primary average reaction. total: " + str(totalReactionTime) + ", amount: " + str(pri_CUE_amount)
        pri_avg_reaction = totalReactionTime / float(pri_CUE_amount)
        #print "calculated primary average reaction: " + str(pri_avg_reaction)
    
    if pri_PE_amount > 0:
        #print "calculating primary error rate 1. cue amount: " + str(pri_CUE_amount) + ", pe amount: " + str(pri_PE_amount)
        pri_err_rate_1 = 1.0 - (float(pri_CUE_amount) / float(pri_PE_amount))
        #print "calculated primary error rate 1: " + str(pri_err_rate_1)
    else:
        pri_err_rate_1 = 1.0
        
    if pri_UE_amount > 0:
        pri_err_rate_2 = 1.0 - (float(pri_CUE_amount) / float(pri_UE_amount))
    else:
        pri_err_rate_2 = 1.0



# if average reaction time has been computed correctly, compute standard deviation
# write reaction times to txt file
if pri_avg_reaction > -0.5:
    pri_sdv_reaction = Util.computeStandardDeviation(reactionTimesList_p,pri_avg_reaction)
    
    for reactionTime in reactionTimesList_p:
        reactionTimesFile_p.write(str(reactionTime) + "\n")





# if there is a secondary task, we need to evaluate it seperately here
# first, make the results we already have, results of the primary task
# then evaluate for secondary task
if isThereSecTask:
    
    isItSecTask1 = True
    if secTaskType == 2: isItSecTask1 = False
    
    secondaryResultSet = Util.searchCorrespondingUserEvent(debug_eval,False,isItSecTask1,secondaryTaskBotEventList,secondaryTaskUserEventList,
                                                           estimatedMaxReactionTime_s,SecTask.START_COLOUR)

    reactionTimesList_s                         = secondaryResultSet[0]
    totalReactionTime_s                         = secondaryResultSet[1]
    amountOfSignificantChanges_s                = secondaryResultSet[2]
    numberOfCorrectlyReactedBotEvents_s         = secondaryResultSet[3]
    numberOfBotEventsNotReactedToCorrectly_s    = secondaryResultSet[4]
    numberOfSignificantChanges                  = secondaryResultSet[5]
    
    sec_CUE_amount      = numberOfCorrectlyReactedBotEvents_s
    sec_IPE_amount      = numberOfBotEventsNotReactedToCorrectly_s
    sec_PE_amount       = numberOfSignificantChanges
    
    
    
    if sec_CUE_amount > 0:
        sec_avg_reaction    = totalReactionTime_s / float(sec_CUE_amount)
    
    if sec_PE_amount > 0:
        sec_err_rate_1      = 1 - (float(sec_CUE_amount) / float(sec_PE_amount))
    else:
        sec_err_rate_1      = 1
    
    
 
    
    sec_event_Pquot = float(sec_PE_amount) / float(sec_PE_amount + pri_PE_amount)     # secondary task: secondary program events / (secPE + priPE)
    
    # if average reaction time has been computed correctly, compute standard deviation
    # write reaction times to txt file
    if sec_avg_reaction > -0.5:
        sec_sdv_reaction = Util.computeStandardDeviation(reactionTimesList_s,sec_avg_reaction)
        
        for reactionTime in reactionTimesList_s:
            reactionTimesFile_s.write(str(reactionTime) + "\n")
    
if pri_avg_reaction <0.0:
    pri_avg_reaction = "N.A."

if sec_avg_reaction <0.0:
    sec_avg_reaction = "N.A."


# write results to file:
print "EVALUATION FINISHED"

if priTaskType == PriTask.JUST_VISUAL_CONT:
    
    pri_PE_amount = roundNumber_p * maxObjectsPt3
    pri_UE_amount = pt3hitCount + pt3errorCount
    pri_CUE_amount = pt3hitCount
    pri_err_rate_2 = pt3errorCount
    
    if pt3errorCount + pt3hitCount > 0:
        pri_err_rate_1 =  float(pt3errorCount) / float(pt3errorCount + pt3hitCount)
    
    hitFile.writerow(hitTimesList)
    errorFile.writerow(errorTimesList)
    
    resultFile.writerow(['user name','task setup','primary task:',
                        'primary task: amount of program events','primary task: amount of user events','primary task: amount of correct user events',
                        'primary task: amount of incorrect user events','primary task: average reaction time','primary task: reaction time standard deviation',
                        'primary task: error rate in relation to all user events','secondary task:','secondary task: amount of program events',
                        'secondary task: amount of correct user events','secondary task: amount of program events not correctly reacted to',
                        'secondary task: average reaction time','secondary task: reaction time standard deviation','secondary task: error rate','secondary program event quota'])
    
    resultFile.writerow([username,taskModeString,
                        str(pri_task_name),str(pri_PE_amount),str(pri_UE_amount),str(pri_CUE_amount),str(pri_err_rate_2),str(pri_avg_reaction),
                        str(pri_sdv_reaction),str(pri_err_rate_1),
                        str(sec_task_name),str(sec_PE_amount),str(sec_CUE_amount),str(sec_IPE_amount),
                        str(sec_avg_reaction),str(sec_sdv_reaction),str(sec_err_rate_1),str(sec_event_Pquot)])
    
    allResultsFile.writerow([username,taskModeString,str(pri_task_name),
                        str(pri_PE_amount),str(pri_UE_amount),str(pri_CUE_amount),"-",str(pri_avg_reaction),
                        str(pri_sdv_reaction),str(pri_err_rate_1),str(pri_err_rate_2),
                        str(sec_task_name),str(sec_PE_amount),str(sec_CUE_amount),str(sec_IPE_amount),
                        str(sec_avg_reaction),str(sec_sdv_reaction),str(sec_err_rate_1),str(sec_event_Pquot)])   


else:    

    resultFile.writerow(['user name','task setup','primary task:',
                        'primary task: amount of program events','primary task: amount of user events','primary task: amount of correct user events',
                        'primary task: amount of program events not correctly reacted to','primary task: average reaction time','primary task: reaction time standard deviation',
                        'primary task: error rate 1','primary task: error rate 2','secondary task:','secondary task: amount of program events',
                        'secondary task: amount of correct user events','secondary task: amount of program events not correctly reacted to',
                        'secondary task: average reaction time','secondary task: reaction time standard deviation','secondary task: error rate','secondary program event quota'])
    
    resultFile.writerow([username,taskModeString,
                        str(pri_task_name),str(pri_PE_amount),str(pri_UE_amount),str(pri_CUE_amount),str(pri_IPE_amount),str(pri_avg_reaction),str(pri_sdv_reaction),
                        str(pri_err_rate_1),str(pri_err_rate_2),str(sec_task_name),str(sec_PE_amount),str(sec_CUE_amount),str(sec_IPE_amount),
                        str(sec_avg_reaction),str(sec_sdv_reaction),str(sec_err_rate_1),str(sec_event_Pquot)])
    
    allResultsFile.writerow([username,taskModeString,
                        str(pri_task_name),str(pri_PE_amount),str(pri_UE_amount),str(pri_CUE_amount),str(pri_IPE_amount),str(pri_avg_reaction),str(pri_sdv_reaction),
                        str(pri_err_rate_1),str(pri_err_rate_2),str(sec_task_name),str(sec_PE_amount),str(sec_CUE_amount),str(sec_IPE_amount),
                        str(sec_avg_reaction),str(sec_sdv_reaction),str(sec_err_rate_1),str(sec_event_Pquot)])

# write results to console
print 'primary task: ' + str(pri_task_name)
print 'primary task: amount of program events: ' + str(pri_PE_amount)
print 'primary task: amount of user events: ' + str(pri_UE_amount)
print 'primary task: amount of correct user events: ' + str(pri_CUE_amount)
print 'primary task: amount of program events not correctly reacted to: ' + str(pri_IPE_amount)
print 'primary task: average reaction time: ' + str(pri_avg_reaction)
print 'primary task: reaction time standard deviation: ' + str(pri_sdv_reaction)
print 'primary task: error rate 1: ' + str(pri_err_rate_1)
print 'primary task: error rate 2: ' + str(pri_err_rate_2)
print 'secondary task: ' + str(sec_task_name)
print 'secondary task: amount of program events: ' + str(sec_PE_amount)
print 'secondary task: amount of correct user events: ' + str(sec_CUE_amount)
print 'secondary task: amount of program events not correctly reacted to: ' + str(sec_IPE_amount)
print 'secondary task: average reaction time: ' + str(sec_avg_reaction)
print 'secondary task: reaction time standard deviation: ' + str(sec_sdv_reaction)
print 'secondary task: error rate: ' + str(sec_err_rate_1)
print 'secondary program event quota: ' + str(sec_event_Pquot)
print 'CPU performance measure: ' + str(performanceMeasure)
print 'PT3 error count ' + str(pt3errorCount)
print 'PT3 hit count ' + str(pt3hitCount)
print 'PT3 rounds (including unfinished): ' + str(roundNumber_p)

pygame.quit()
sys.exit()