import serial, threading,re
 
 

class SerialManager(threading.Thread):
    ser = serial.Serial()
    usable = False
    lineRead = 0
    lineAmount = 0
    debug = False
    runStatus = True
    
    def __init__(self, debugPar):
        threading.Thread.__init__(self)
        self.debug = debugPar
        if self.debug: print "Serial Manager initialized"
        
    # writes a number as a byte to Ambient Presence Indicator.
    def write(self, i):
        if self.usable:
            output = bytearray([i])
            self.ser.write(output)

    def setPort(self,i):
        self.ser.port = i
        self.ser.open()
        self.usable = True
    
    # this is an own thread. there is no other way to implement reading from serial port without blocking
    # reads data from Ambient Presence Indicator, recognizes, writes into lineRead and counts up lineAmount
    # so other functions know there is new data to receive
    def run(self):
        while self.runStatus:
            data = self.ser.read(13)
            if self.debug: print "read: " + str(data)
            
            pattern0 = re.compile("Button")
            pattern1 = re.compile("Status='1")
            pattern2 = re.compile("Status='-")
        
            # button has been pressed
            if pattern0.match(data):
                data = ""
                
            # main light has been turned in "positive" direction
            elif  pattern1.match(data):
                self.lineRead = 1
                if self.debug: print "Positive turn recognized!"
                self.lineAmount = self.lineAmount +1
                
            # main light has been turned in "negative" direction
            elif pattern2.match(data):
                self.lineRead = -1
                self.lineAmount = self.lineAmount +1
                if self.debug: print "Negative turn recognized!"
                data = self.ser.read(1)
                
            # data not recognized or serial manager is being closed
            else:
                print "DATA NOT RECOGNIZED! Maybe Serial Connection was closed because the test is over..."
            
            
            
    def getLineAmount(self):
        return self.lineAmount
    
    def getLastLine(self):
        return self.lineRead
    
    def closeSerial(self):
        self.runStatus = False
        self.ser.close()
        