import SecTask
from statlib import stats


# task modes
global TASK_MODE_A, TASK_MODE_B, TASK_MODE_C, TASK_MODE_D, TASK_MODE_E, TASK_MODE_F
global BASELINE_MODE_1, BASELINE_MODE_3
global TRAINING_MODE_A, TRAINING_MODE_B, TRAINING_MODE_C, TRAINING_MODE_D, TRAINING_MODE_E, TRAINING_MODE_F
global TRAINING_MODE_1, TRAINING_MODE_2, TRAINING_MODE_3

TASK_MODE_A = 1
TASK_MODE_B = 2
TASK_MODE_C = 3
TASK_MODE_D = 4
TASK_MODE_E = 5
TASK_MODE_F = 6
BASELINE_MODE_1 = 7
BASELINE_MODE_3 = 8
TRAINING_MODE_A = 11
TRAINING_MODE_B = 12
TRAINING_MODE_C = 13
TRAINING_MODE_D = 14
TRAINING_MODE_E = 15
TRAINING_MODE_F = 16
TRAINING_MODE_1 = 21
TRAINING_MODE_2 = 22
TRAINING_MODE_3 = 23



def computeAverage(parValueList):
    tempResult = 0.0
    for value in parValueList:
        tempResult = tempResult + value
    if len(parValueList) > 0:
        tempResult = tempResult / float(len(parValueList))
        
    return tempResult



def computeStandardDeviation(parReactionTimesList,parAverageReactionTime):
    if len(parReactionTimesList) > 0:
        return stats.samplestdev(parReactionTimesList)
    else:
        return "N.A."


# sort out all user input events that occured before the first program event appeared
# (they can't be correct)
def sortOutEarlyEvents(parAppearance,parUserEventList):
    
    #print "sorting out events before " + str(parAppearance)
    
    tempList = parUserEventList
    firstAppearance = parAppearance # time of the first program event
    i4 = 0
    appearanceReached = False   # has the first program event been reached?

    # count how many have been counted out for debug
    sortOutCounter = 0
    
    while (i4 < len(tempList) and (not appearanceReached)):
        
        #print "-- tempelement's time is " + str(float(tempList[i4][0]))
        
        if float(tempList[i4][0]) < firstAppearance:
            #print "-- it is being trued!"
            tempList[i4][2] = True
            sortOutCounter = sortOutCounter +1
        else:
            #print "-- done with sorting out early events!"
            appearanceReached = True
        i4 = i4 +1
    
    
    #print "-- sorted out " + str(sortOutCounter) + " of " + str(len(tempList))
    
    return tempList



# go through all program events and search for a corresponding user event
# this works for primary as well as for secondary tasks
def searchCorrespondingUserEvent(debug,isItPrimaryTask,isItSecTask1,botEventListPar,userEventListPar,estimatedMaxReactionTime,maxErrorsOrStartColour):
    
    
    if debug: print "searchCorrUserEvent called with " + str(debug) + ", " + str(isItPrimaryTask) + ", " + str(isItSecTask1) + ", " + str(botEventListPar) + ", " + str(userEventListPar) + ", " + str(estimatedMaxReactionTime) + ", " + str(maxErrorsOrStartColour)
    
    
    botEventList = botEventListPar
    userEventList = userEventListPar
    numberOfCorrectlyReactedBotEvents = 0
    numberOfSignificantChanges = 0
    resultString =""
    
    
    # PRIMARY EVALUATION
    if isItPrimaryTask:
        
        if debug: print "IT IS PRIMARY EVALUATION TIME!"
        numberOfBotEventsNotReactedToCorrectly = 0
        reactionTimesList = []
        totalReactionTime = 0
        incorrectEventsFound = 0
        
        
        # go through all program events
        for i5 in range(0,len(botEventList)):
            
            if debug: print "PRIMARY EVALUATION CHECK " + str(i5) + ", " + str(botEventList[i5]) 
            
            incorrectEventsFound = 0
            
            earliestTime = float(botEventList[i5][0])
            latestTime = earliestTime + estimatedMaxReactionTime
            i6=0
            eventAmount = len(userEventList)
            userEventFound = False
            # go through user events, sort out checked ones and those with time that don't fit
            while((i6 < eventAmount) and (not userEventFound) and (incorrectEventsFound <= maxErrorsOrStartColour)):
                
                if debug: print "-PRIMARY EVALUATION CHECK " + str(i5) + " while loop with " + str(i6) + ", " + str(userEventList[i6])
                
                #if user event has not been checked yet
                if (not userEventList[i6][2]):
                    tempEventTime = userEventList[i6][0]
                    if debug: print "--PRIMARY EVALUATION CHECK " + str(i5) + " while loop with " + str(i6) +" not checked yet. time is " + str(tempEventTime) + " against earliest " + str(earliestTime) + " and latest " + str(latestTime)
                    
                    #if user event is in time frame: count up the cancel counter
                    if tempEventTime > earliestTime and tempEventTime < latestTime:
                        if debug: print "--PRIMARY EVALUATION CHECK " + str(i5) + " while loop with " + str(i6) +" not checked yet and in time frame"
                        incorrectEventsFound = incorrectEventsFound + 1
                        
                        #if user event is of same type as program event
                        if (botEventList[i5][1] == userEventList[i6][1]):
                            userEventList[i6][2] = True
                            userEventList[i6][3] = True
                            botEventList[i5][4] = True
                            userEventFound = True
                            numberOfCorrectlyReactedBotEvents = numberOfCorrectlyReactedBotEvents +1
                            tempReactionTime = tempEventTime - float(botEventList[i5][0])
                            reactionTimesList.append(tempReactionTime)
                            totalReactionTime = totalReactionTime + tempReactionTime
                            resultString += "Program event number " + str(i5) + " reacted to after " + str(tempReactionTime) + "\n"
                            if debug: print "PRIMARY EVALUATION CHECK " + str(i5) + " success"
                i6 = i6+1
            if not userEventFound:
                if debug: print "PRIMARY EVALUATION CHECK " + str(i5) + " failed"
                numberOfBotEventsNotReactedToCorrectly = numberOfBotEventsNotReactedToCorrectly + 1
        if debug: print "RESULTS FROM PRIMARY EVAL:\n--------\n" + str(resultString) + "\n------------\n"
        return [reactionTimesList,totalReactionTime,resultString,incorrectEventsFound,numberOfCorrectlyReactedBotEvents,numberOfBotEventsNotReactedToCorrectly]
    
    
    # SECONDARY EVALUATION
    else:
        
        # it is secTask1: turn main light to colour that most lights have
        if isItSecTask1:
            # return values
            reactionTimesList = []
            totalReactionTime = 0
            amountOfSignificantChanges = 0
            numberOfCorrectlyReactedBotEvents = 0
            numberOfBotEventsNotReactedToCorrectly = 0
            
            # other values
            significantChangesList = []
            dominantColour = maxErrorsOrStartColour
            lightStates = [99,maxErrorsOrStartColour,maxErrorsOrStartColour,maxErrorsOrStartColour,maxErrorsOrStartColour]
            resultString = ""
            
            
            # fill a new list only with the program events that matter ( = significant changes)
            for runMemberOfBotEventList in botEventList:
                eventCode = int(runMemberOfBotEventList[1])
                whichLight =  eventCode / 100
                newColour = eventCode % 100
                lightStates[whichLight] = newColour
                newDominantColour = computeDominantColour(lightStates)
                if newDominantColour != dominantColour:
                    significantChangesList.append([runMemberOfBotEventList,newDominantColour])
                    dominantColour = newDominantColour
                    amountOfSignificantChanges = amountOfSignificantChanges +1
                    
            if debug: print "SECONDARY1 EVAL Number of significant changes: " + str(amountOfSignificantChanges) + "/" + str(len(significantChangesList))
                    
            # search through list, find out if there was a proper reaction, and how long it took to give the proper reaction
            for runMemberOfSigChangList in significantChangesList:
                earliestTime = float(runMemberOfSigChangList[0][0])
                latestTime = earliestTime + estimatedMaxReactionTime
                
                
                i6=0
                eventAmount = len(userEventList)
                userEventFound = False
                reachedEndOfTimeFrame = False
                
                
                listOfUncheckedUserEventsInTimeFrame = []
                
                               
                if debug: print "Searching matching user event for significant change number " + str(runMemberOfSigChangList)
                
                # fill unchecked events from time frame into list
                while( (i6 < eventAmount) and (not userEventFound) and (not reachedEndOfTimeFrame)):
                
                    #if user event has not been checked yet
                    if (not userEventList[i6][2]):
                        tempEventTime = float(userEventList[i6][0])
                        
                        #if user event is in time frame
                        if tempEventTime > earliestTime and tempEventTime < latestTime:
                            
                            newColour = int(userEventList[i6][1]) - 10
                            
                            listOfUncheckedUserEventsInTimeFrame.append([tempEventTime,newColour])
                        elif tempEventTime > latestTime:
                            reachedEndOfTimeFrame = True
                            
                            
                    i6 = i6+1
                
                #correction for while loop parameter. we reuse that parameter because we use the "last" event list entry
                i6 = i6-1
                    
                # if events are found...    
                
                amountOfUncheckedEventsInTimeFrame = len(listOfUncheckedUserEventsInTimeFrame)
                if debug: print "- unchecked events in time frame filled into list: " + str(amountOfUncheckedEventsInTimeFrame)
                
                if (amountOfUncheckedEventsInTimeFrame > 0):
                    if debug: print "--" + str(listOfUncheckedUserEventsInTimeFrame)
                    lastEvent = listOfUncheckedUserEventsInTimeFrame[len(listOfUncheckedUserEventsInTimeFrame)-1]
                    
                    # if event has correct type
                    if (runMemberOfSigChangList[1] == lastEvent[1]):
                        userEventList[i6][2] = True
                        userEventList[i6][3] = True
                        userEventFound = True
                        numberOfCorrectlyReactedBotEvents = numberOfCorrectlyReactedBotEvents +1
                        tempReactionTime = lastEvent[0] - float(runMemberOfSigChangList[0][0])
                        reactionTimesList.append(tempReactionTime)
                        totalReactionTime = totalReactionTime + tempReactionTime
                            
                        if debug: print "-- last user event " + str(lastEvent) + " has matched with reaction time: " + str(tempReactionTime)
                        resultString += "Program event number " + str(runMemberOfSigChangList) + " reacted to after " + str(tempReactionTime) + "\n"
                    else:
                        if debug: print "-- last user event has not matched! user event: " + str(lastEvent) + ", program event: " + str(runMemberOfSigChangList)
                else:
                    if debug: print "No unchecked events in time frame found!"
        
                    
                if not userEventFound: numberOfBotEventsNotReactedToCorrectly = numberOfBotEventsNotReactedToCorrectly + 1
            numberOfSignificantChanges = len(significantChangesList) 
            return [reactionTimesList, totalReactionTime, amountOfSignificantChanges, numberOfCorrectlyReactedBotEvents,
                    numberOfBotEventsNotReactedToCorrectly, numberOfSignificantChanges]
              
              
              
              
        # it is secTask2: turn main light to colour that light nr X has        
        else:
            # return values
            reactionTimesList = []
            totalReactionTime = 0
            amountOfSignificantChanges = 0
            numberOfCorrectlyReactedBotEvents = 0
            numberOfBotEventsNotReactedToCorrectly = 0
            
            # other values
            significantChangesList = []
            relevantLightColour = maxErrorsOrStartColour
            
            # fill a new list only with the events that matter
            for runMemberOfBotEventList in botEventList:
                eventCode = int(runMemberOfBotEventList[1])
                whichLight =  eventCode / 100
                newColour = eventCode % 100
                if whichLight == SecTask.TASK_2_RELEVANT_LIGHT_NR:
                    relevantLightColour = newColour
                    significantChangesList.append([runMemberOfBotEventList,relevantLightColour])
                    amountOfSignificantChanges = amountOfSignificantChanges +1
                    
                 
            if debug: print "about to enter sigchangeslist"
               
            # search through list, find out if there was a proper reaction, and how long it took to give the proper reaction
            for runMemberOfSigChangList in significantChangesList:
                earliestTime = float(runMemberOfSigChangList[0][0])
                latestTime = earliestTime + estimatedMaxReactionTime
                
                
                i6=0
                eventAmount = len(userEventList)
                userEventFound = False
                reachedEndOfTimeFrame = False
                
                listOfUncheckedUserEventsInTimeFrame = []
                
                               
                if debug: print "Searching matching user event for significant change number " + str(runMemberOfSigChangList) + ", earliest " + str(earliestTime) + ", latest " + str(latestTime)
                
                # fill unchecked events from time frame into list
                while( (i6 < eventAmount) and (not userEventFound) and (not reachedEndOfTimeFrame)):
                
                    if debug: print "- User event nr " + str(i6) + " is: " + str(userEventList[i6])
                    #if user event has not been checked yet
                    if (not userEventList[i6][2]):
                        tempEventTime = float(userEventList[i6][0])
                        
                        #if user event is in time frame
                        if tempEventTime > earliestTime and tempEventTime < latestTime:
                            
                            newColour = int(userEventList[i6][1]) - 10
                            
                            listOfUncheckedUserEventsInTimeFrame.append([tempEventTime,newColour])
                        elif tempEventTime > latestTime:
                            reachedEndOfTimeFrame = True
                            
                            
                    i6 = i6+1
                   
                   
                #correction for while loop parameter. we reuse that parameter because we use the "last" event list entry
                i6 = i6-1
                 
                # if events are found...    
                
                amountOfUncheckedEventsInTimeFrame = len(listOfUncheckedUserEventsInTimeFrame)
                if debug: print "- unchecked events in time frame filled into list: " + str(amountOfUncheckedEventsInTimeFrame)
                
                if (amountOfUncheckedEventsInTimeFrame > 0):
                    lastEvent = listOfUncheckedUserEventsInTimeFrame[len(listOfUncheckedUserEventsInTimeFrame)-1]
                    
                    # if event type matches
                    if (runMemberOfSigChangList[1] == lastEvent[1]):
                        userEventList[i6][2] = True
                        userEventList[i6][3] = True
                        userEventFound = True
                        numberOfCorrectlyReactedBotEvents = numberOfCorrectlyReactedBotEvents +1
                        tempReactionTime = lastEvent[0] - float(runMemberOfSigChangList[0][0])
                        reactionTimesList.append(tempReactionTime)
                        totalReactionTime = totalReactionTime + tempReactionTime
                            
                        if debug: print "-- last user event has matched with reaction time: " + str(tempReactionTime)
                        resultString += "Program event number " + str(runMemberOfSigChangList) + " reacted to after " + str(tempReactionTime) + "\n"
                    else:
                        if debug: print "-- last user event has not matched! user event: " + str(lastEvent) + ", program event: " + str(runMemberOfSigChangList)
                else:
                    if debug: print "No unchecked events in time frame found!"
                    
                    
                if not userEventFound: numberOfBotEventsNotReactedToCorrectly = numberOfBotEventsNotReactedToCorrectly + 1
            numberOfSignificantChanges = len(significantChangesList) 
            return [reactionTimesList, totalReactionTime, amountOfSignificantChanges, numberOfCorrectlyReactedBotEvents,
                    numberOfBotEventsNotReactedToCorrectly, numberOfSignificantChanges]

# this is for primary task 1
# find out which colour is the dominant one
def computeDominantColour(lightStates):
    tempColour = 99
    redCount = 0
    greenCount = 0
    blueCount = 0
    for i in range(1,5):
                    
        if lightStates[i] == SecTask.RED: redCount = redCount +1
        elif lightStates[i] == SecTask.GREEN: greenCount = greenCount +1
        elif lightStates[i] == SecTask.BLUE: blueCount = blueCount +1
        
    
    if redCount > 1: tempColour = SecTask.RED
    elif greenCount > 1: tempColour = SecTask.GREEN
    elif blueCount > 1: tempColour = SecTask.BLUE
    else: print "ERROR AT UTIL.COMPUTEDOMINANTCOLOUR. red: " + str(redCount) + ", green: " + str(greenCount) + ", blue: " + str(blueCount)
    
    return tempColour

# for main, builds a string that will be the foldername for saving txt files and results
def buildFoldernameString(now,username,taskModeStringPar):
    month = now.month
    day = now.day
    hour = now.hour
    minute = now.minute
    second = now.second
    
    if month<10:
        month = "0" + str(month)
    else:
        month = str(month)
        
    if day<10:
        day = "0" + str(day)
    else:
        day = str(day)
        
    if hour<10:
        hour = "0" + str(hour)
    else:
        hour = str(hour)
        
    if minute<10:
        minute = "0" + str(minute)
    else:
        minute = str(minute)
        
    if second<10:
        second = "0" + str(second)
    else:
        second = str(second)
        
    return "txt/" + username + "_-_" + str(now.year) + "-" + month + "-" + day + "_" + hour + "." + minute + "." + second + "_-_" + str(taskModeStringPar)
